#include <vector>
#include <iostream>

#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
#include <CGAL/convex_hull_3.h>

typedef CGAL::Exact_predicates_exact_constructions_kernel  Kernel;
typedef Kernel::Point_3                                    Point;
typedef Kernel::Plane_3                                    Plane;
typedef Kernel::Vector_3                                   Vector;

typedef Kernel                                          Traits;
typedef CGAL::Polyhedron_3<Traits>                      Polyhedron;
typedef Polyhedron::Vertex                              Vertex;

/*! Transform a (planar) facet into a plane */
struct Plane_equation {
  template <class Facet>
  typename Facet::Plane operator()(Facet & f) {
    typename Facet::Halfedge_handle h = f.halfedge();
    typedef typename Facet::Plane Plane;
    return Plane( h->vertex()->point(),
                  h->next()->vertex()->point(),
                  h->next()->next()->vertex()->point());
  }
};

int main(int argc, char * argv[])
{
  std::vector<Point> in1, in2;
  in1.resize(8);
  in1[0] = Point(-1, -1, -1);
  in1[1] = Point(1, -1, -1);
  in1[2] = Point(1, 1, -1);
  in1[3] = Point(-1, 1, -1);
  in1[4] = Point(-1, -1, 1);
  in1[5] = Point(1, -1, 1);
  in1[6] = Point(1, 1, 1);
  in1[7] = Point(-1, 1, 1);

  in2.resize(4);
  in2[0] = Point(-1, -1, -1);
  in2[1] = Point(0, 1, 1);
  in2[2] = Point(1, -1, -1);
  in2[3] = Point(0, 1, 0);
  
  std::vector<Point> points(in1.size() * in2.size());

  std::vector<Point>::const_iterator it1, it2;
  std::vector<Point>::iterator it3 = points.begin();
  for (it1 = in1.begin(); it1 != in1.end(); ++it1) {
    Vector v(CGAL::ORIGIN, *it1);
    for (it2 = in2.begin(); it2 != in2.end(); ++it2) *it3++ = (*it2) + v;
  }

#if 1
  std::copy(points.begin(), points.end(),
            std::ostream_iterator<Point>(std::cout, "\n"));
#endif

  Polyhedron polyhedron;
  CGAL::convex_hull_3(points.begin(), points.end(), polyhedron);
  std::transform(polyhedron.facets_begin(), polyhedron.facets_end(),
                 polyhedron.planes_begin(), Plane_equation());
  return 0;
}
